@@ -5,7 +5,7 @@
 
 #if defined __aarch64__
 # include "tdep-aarch64/jmpbuf.h"
-#if defined __arm__
+#elif defined __arm__
 # include "tdep-arm/jmpbuf.h"
 #elif defined __hppa__
 # include "tdep-hppa/jmpbuf.h"
