@@ -1,6 +1,6 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2001-2002 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2001-2002, 2005 Hewlett-Packard Co
+        Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
 
@@ -23,16 +23,13 @@ LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
-#include <stdio.h>
-
-#include "internal.h"
-#include "tdep.h"
+#include "libunwind_i.h"
 
 #ifdef UNW_REMOTE_ONLY
 
 static inline int
 local_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
-		      int need_unwind_info, void *arg)
+                      int need_unwind_info, void *arg)
 {
   return -UNW_ENOINFO;
 }
@@ -41,7 +38,7 @@ local_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
 
 static inline int
 local_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
-		      int need_unwind_info, void *arg)
+                      int need_unwind_info, void *arg)
 {
   unw_dyn_info_list_t *list;
   unw_dyn_info_t *di;
@@ -52,11 +49,11 @@ local_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
     return -UNW_ENOINFO;
 #endif
 
-  list = (unw_dyn_info_list_t *) _U_dyn_info_list_addr ();
+  list = (unw_dyn_info_list_t *) (uintptr_t) _U_dyn_info_list_addr ();
   for (di = list->first; di; di = di->next)
     if (ip >= di->start_ip && ip < di->end_ip)
       return unwi_extract_dynamic_proc_info (as, ip, pi, di, need_unwind_info,
-					     arg);
+                                             arg);
   return -UNW_ENOINFO;
 }
 
@@ -66,7 +63,7 @@ local_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
 
 static inline int
 remote_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
-		       int need_unwind_info, void *arg)
+                       int need_unwind_info, void *arg)
 {
   return -UNW_ENOINFO;
 }
@@ -75,7 +72,7 @@ remote_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
 
 static inline int
 remote_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
-		       int need_unwind_info, void *arg)
+                       int need_unwind_info, void *arg)
 {
   return unwi_dyn_remote_find_proc_info (as, ip, pi, need_unwind_info, arg);
 }
@@ -84,8 +81,8 @@ remote_find_proc_info (unw_addr_space_t as, unw_word_t ip, unw_proc_info_t *pi,
 
 HIDDEN int
 unwi_find_dynamic_proc_info (unw_addr_space_t as, unw_word_t ip,
-			     unw_proc_info_t *pi, int need_unwind_info,
-			     void *arg)
+                             unw_proc_info_t *pi, int need_unwind_info,
+                             void *arg)
 {
   if (as == unw_local_addr_space)
     return local_find_proc_info (as, ip, pi, need_unwind_info, arg);
