@@ -25,6 +25,8 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 # include "config.h"
 #endif
 
+#include "compiler.h"
+
 #include <errno.h>
 #if HAVE_EXECINFO_H
 # include <execinfo.h>
@@ -35,33 +37,29 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ucontext.h>
 #include <unistd.h>
 #include <libunwind.h>
 
-#if UNW_TARGET_X86 || UNW_TARGET_X86_64 || UNW_TARGET_ARM
-# define STACK_SIZE	(128*1024)	/* On x86/-64 and ARM, SIGSTKSZ is too small */
-#else
-# define STACK_SIZE	SIGSTKSZ
-#endif
-
 #define panic(args...)				\
 	{ fprintf (stderr, args); exit (-1); }
 
-#ifndef HAVE_SIGHANDLER_T
-typedef RETSIGTYPE (*sighandler_t) (int);
-#endif
+#define SIG_STACK_SIZE 0x100000
 
 int verbose;
 int num_errors;
 
+/* These variables are global because they
+ * cause the signal stack to overflow */
+char buf[512], name[256];
+unw_cursor_t cursor;
+unw_context_t uc;
+
 static void
 do_backtrace (void)
 {
-  char buf[512], name[256];
   unw_word_t ip, sp, off;
-  unw_cursor_t cursor;
   unw_proc_info_t pi;
-  unw_context_t uc;
   int ret;
 
   if (verbose)
@@ -87,10 +85,12 @@ do_backtrace (void)
 	{
 	  printf ("%016lx %-32s (sp=%016lx)\n", (long) ip, buf, (long) sp);
 
-	  unw_get_proc_info (&cursor, &pi);
-	  printf ("\tproc=%lx-%lx\n\thandler=%lx lsda=%lx gp=%lx",
+	  if (unw_get_proc_info (&cursor, &pi) == 0)
+	    {
+	      printf ("\tproc=0x%lx-0x%lx\n\thandler=0x%lx lsda=0x%lx gp=0x%lx",
 		  (long) pi.start_ip, (long) pi.end_ip,
 		  (long) pi.handler, (long) pi.lsda, (long) pi.gp);
+	    }
 
 #if UNW_TARGET_IA64
 	  {
@@ -128,7 +128,7 @@ do_backtrace (void)
 }
 
 void
-foo (long val)
+foo (long val UNUSED)
 {
   do_backtrace ();
 }
@@ -165,11 +165,13 @@ bar (long v)
 }
 
 void
-sighandler (int signal, void *siginfo, void *context)
+sighandler (int signal, void *siginfo UNUSED, void *context)
 {
-  ucontext_t *uc = context;
+  ucontext_t *uc UNUSED;
   int sp;
 
+  uc = context;
+
   if (verbose)
     {
       printf ("sighandler: got signal %d, sp=%p", signal, &sp);
@@ -187,7 +189,17 @@ sighandler (int signal, void *siginfo, void *context)
       }
 # endif
 #elif UNW_TARGET_X86
+#if defined __linux__
       printf (" @ %lx", (unsigned long) uc->uc_mcontext.gregs[REG_EIP]);
+#elif defined __FreeBSD__
+      printf (" @ %lx", (unsigned long) uc->uc_mcontext.mc_eip);
+#endif
+#elif UNW_TARGET_X86_64
+#if defined __linux__
+      printf (" @ %lx", (unsigned long) uc->uc_mcontext.gregs[REG_RIP]);
+#elif defined __FreeBSD__
+      printf (" @ %lx", (unsigned long) uc->uc_mcontext.mc_rip);
+#endif
 #endif
       printf ("\n");
     }
@@ -195,7 +207,7 @@ sighandler (int signal, void *siginfo, void *context)
 }
 
 int
-main (int argc, char **argv)
+main (int argc, char **argv UNUSED)
 {
   struct sigaction act;
   stack_t stk;
@@ -219,10 +231,10 @@ main (int argc, char **argv)
 
   if (verbose)
     printf ("\nBacktrace across signal handler on alternate stack:\n");
-  stk.ss_sp = malloc (STACK_SIZE);
+  stk.ss_sp = malloc (SIG_STACK_SIZE);
   if (!stk.ss_sp)
-    panic ("failed to allocate SIGSTKSZ (%u) bytes\n", SIGSTKSZ);
-  stk.ss_size = STACK_SIZE;
+    panic ("failed to allocate %u bytes\n", SIG_STACK_SIZE);
+  stk.ss_size = SIG_STACK_SIZE;
   stk.ss_flags = 0;
   if (sigaltstack (&stk, NULL) < 0)
     panic ("sigaltstack: %s\n", strerror (errno));
@@ -241,5 +253,11 @@ main (int argc, char **argv)
     }
   if (verbose)
     printf ("SUCCESS.\n");
+
+  signal (SIGTERM, SIG_DFL);
+  stk.ss_flags = SS_DISABLE;
+  sigaltstack (&stk, NULL);
+  free (stk.ss_sp);
+
   return 0;
 }
